<?php

namespace App\Http\Controllers;

use App\Models\Student;
use App\Models\User;
use App\Repositories\StudentGuardianRepository;
use App\Repositories\UserRepository;
use Illuminate\Http\Request;
use App\Repositories\StudentRepository;
use App\Repositories\StudentByCourseRepository;
use App\Repositories\StudentOptionalSubjectRepository;
use App\Repositories\SchoolYearRepository;
use App\Repositories\QualitativeEquivalenceRepository;
use App\Repositories\CourseRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

class StudentController extends BaseController {

    protected $student;
    protected $school_year;
    protected $student_by_course;
    protected $student_optional_subject;
    protected $qualitativeEquivalence;
    protected $course;
    protected $user;
    protected $studentGuardian;

    public function __construct(StudentRepository $student, SchoolYearRepository $school_year, StudentByCourseRepository $student_by_course, StudentOptionalSubjectRepository $student_optional_subject, QualitativeEquivalenceRepository $qualitativeEquivalence, CourseRepository $course, UserRepository $user, StudentGuardianRepository $studentGuardian) {
        $this->student = $student;
        $this->school_year = $school_year;
        $this->student_by_course = $student_by_course;
        $this->student_optional_subject = $student_optional_subject;
        $this->qualitativeEquivalence = $qualitativeEquivalence;
        $this->course = $course;
        $this->user = $user;
        $this->studentGuardian = $studentGuardian;
    }

    /**
     * Método para obtener datos de estudiante por serial
     * @since   2018-04-07
     * @param   integer serial_std serial estudiante
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getStudentDataBySerialStd($serial_std) {

        $student = $this->student->getStudentBySerialStd($serial_std);

        return response()->json($student);
    }

    /**
     * Método para obtener las materias y calificaciones del estudiante
     * @since   2018-05-17
     * @param   integer serial_prd serial del periodo
     * @param integer $serial_std serial del estudiante
     * @author   Fernando Salas <rfsalas@rutatec.com><br /><br/>
     * @return  json $subjects
     *
     */
    public function getSubjectsByStudent($serial_prd, $serial_std) {
        if ($serial_prd == 0) {
            return response()->json(false);
        }
        $subjects = $this->student->getSubjectsByPeriod($serial_prd, $serial_std);
		$period = \App\Models\Period::find($serial_prd);
        $periodGroupYear = $period->periodGroupYear;
        foreach ($subjects as &$sbj) {
            if ($sbj->GRADETYPE == 'QUALITATIVE') {
                foreach ($sbj['attributes'] as $key => $value) {
                    if ($key != 'SERIAL' && $key != 'MATERIA' && $key != 'GRADETYPE' && $key != 'TYPE') {
                        $sbj->$key = $this->qualitativeEquivalence->convertQuantitativeToQualitative($value, $periodGroupYear->serial_scy);
                    }
                }
            }
            $sbj['PERIOD'] = $serial_prd;
        }
        return response()->json($subjects);
    }

    public function getSubjectsYearByStudent($serial_std) {
        $currentSchoolYear = $this->school_year->getCurrentSchoolYear();
        $studentGuardian  = $this->studentGuardian->getUserGuardian($serial_std,'INACTIVE');
        if (count($studentGuardian)) {
            return response()->json(false);
        } else {
            if (isset($currentSchoolYear->serial_scy)) {
                $subjects = $this->student->getSubjectsByStudentYear($serial_std, $currentSchoolYear->serial_scy);
                foreach ($subjects as $sbj) {
                    if ($sbj->GRADETYPE == 'QUALITATIVE') {
                        foreach ($sbj['attributes'] as $key => $value) {
                            if ($key != 'SERIAL' && $key != 'MATERIA' && $key != 'GRADETYPE' && $key != 'TYPE') {
                                $sbj->$key = $this->qualitativeEquivalence->convertQuantitativeToQualitative($value, $currentSchoolYear->serial_scy);
                            }
                        }
                    }
                }
                return response()->json($subjects);
            } else {
                return response()->json(false);
            }
        }
    }

    /**
     * Método que devuelve datos para el carnet de un estudiante.
     * @since   2018-06-21
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @param   integer $serial_std
     * @return  json $students
     *
     */
    public function getStudentCard($serial_std) {
        $school_year = $this->school_year->getCurrentSchoolYear();
        $course = $this->course->getCourseByStudentYear($serial_std, $school_year->serial_scy);
        $infoCourse = $this->course->getCourseDescription($course->serial_crs);
        $serial_usr = User::where('serial_std', '=', $serial_std)->first();
        $photo = $this->user->getImageUser($serial_usr->serial_usr);
        $student = $this->student->findById($serial_std);
        $student = [
            'serial_std' => $student->serial_std,
            'first_name_std' => $student->first_name_std,
            'last_name_std' => $student->last_name_std,
            'photo_usr' => $photo,
            //'course' => $infoCourse->name_lvl . ' DE ' . $infoCourse->name_set . ' ' . $infoCourse->name_clg,
            'course' => $infoCourse->name_lvl . ' DE ' . $infoCourse->name_set,
            'name_scy' => $school_year->name_scy,
            'start_date_scy' => $school_year->start_date_scy,
            'end_date_scy' => $school_year->end_date_scy
        ];
        return response()->json($student);
    }

    /**
     * Método que devuelve los padres de un estudiante.
     * @since   2018-06-25
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @param   integer $serial_std
     * @return  json $students
     *
     */
    public function getGuardiansStudent($serial_std) {
        $student = Student::find($serial_std);
        $guardians = $student->guardians;
        $grds = array();
        foreach ($guardians as $guardian) {
            $guardian = [
                'first_name_grd' => $guardian->first_name_grd,
                'last_name_grd' => $guardian->last_name_grd
            ];
            array_push($grds, $guardian);
        }
        return response()->json($grds);
    }

    /**
     * Método que devuelve representantes por estudiante
     * @since   2018-07-24  
     * @param   integer serial_std serial_std
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getGuardiansBySerialStd($serial_std) {
        $guardians = $this->student->getGuardiansBySerialStd($serial_std);
        return response()->json($guardians);
    }

}
