<?php

namespace App\Repositories;

use App\Models\Course;
use App\Models\OptionalSubject;
use App\Repositories\BaseRepository;

class CourseRepository extends BaseRepository
{

    protected $modelClass = Course::class;
    protected $course;

    //Declaración del constructor
    public function __construct(Course $course)
    {
        $this->course = $course;
    }

    /**
     * Método para devolver los estudiantes del curso
     * @since   2017-08-03
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   integer $serial_crs serial del curso
     * @return  object $students
     *
     * @edit    2017-08-03<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getStudents($serial_crs)
    {
        $query = $this->newQuery();
        $query->join('student_by_course', function ($join) use ($serial_crs) {
            $join->on('course.serial_crs', '=', 'student_by_course.serial_crs');
            $join->where('student_by_course.serial_crs', '=', $serial_crs);
            $join->where('student_by_course.status_stc', '=', 'ACTIVE');
        });
        $query->join('student', function ($join) {
            $join->on('student_by_course.serial_std', '=', 'student.serial_std');
            $join->where('student.status_std', '=', 'ACTIVE');
        });
        $query->join('user', function ($join) {
            $join->on('student.serial_std', '=', 'user.serial_std');
        });
        $query->orderBy('student.last_name_std', 'ASC');
        $query->orderBy('student.first_name_std', 'ASC');

        return $this->doQueryGet($query);
    }

    /**
     * Método que devuelve el curso
     * @since   2017-08-03
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   integer $serial_crs serial del curso
     * @return  object $course
     *
     * @edit    2017-08-03<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getCourse($serial_crs)
    {
        $query = $this->newQuery();
        $query->where('serial_crs', '=', $serial_crs);
        $query->where('status_crs', '=', 'ACTIVE');
        return $this->doQueryFirst($query);
    }


    /**
     * Método que devuelve el curso de un estudiante por serial_std y anio
     * @since   2018-04-02
     * @author Erick Vaca <epvaca@rutatec.com>
     * @param   integer $serial_std serial del estudiante
     * @param   integer $serial_scy serial del anio
     * @return  object $course
     *
     * @edit    2018-04-02<br />
     *          Erick Vaca <epvaca@rutatec.com> <br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getCourseByStudentYear($serial_std, $serial_scy)
    {
        $query = $this->newQuery();

        $query->join('student_by_course as sbc', 'sbc.serial_crs', '=', 'course.serial_crs');
        $query->where('serial_std', '=', $serial_std);
        $query->where('serial_scy', '=', $serial_scy);
        $query->where('status_crs', '=', 'ACTIVE');
        $query->where('status_stc', '=', 'ACTIVE');
        $query->orderBy('date_start_sbc', 'DESC');

        return $this->doQueryFirst($query);
    }

    /**
     * Método que devuelve datos para el carnet de un estudiante.
     * @since   2018-06-21
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @param   integer $serial_std
     * @return  json $students
     *
     */

    public function getCourseDescription($serial_crs)
    {
        $query = $this->newQuery();
        $query->join('section', function ($join) use ($serial_crs) {
            $join->on('course.serial_set', '=', 'section.serial_set');
            $join->where('course.serial_crs', '=', $serial_crs);
        });
        $query->join('level', function ($join) use ($serial_crs) {
            $join->on('course.serial_lvl', '=', 'level.serial_lvl');
            $join->where('course.serial_crs', '=', $serial_crs);
        });
        $query->join('class_group', function ($join) use ($serial_crs) {
            $join->on('course.serial_clg', '=', 'class_group.serial_clg');
            $join->where('course.serial_crs', '=', $serial_crs);
        });

        return $this->doQueryFirst($query);
    }

    public function getAcademicArea($serial_scy,$serial_set,$serial_lvl){
        $query = $this->newQuery();
        $query->join('class_group', function ($join) {
            $join->on('course.serial_clg', '=', 'class_group.serial_clg');
        });
        $query->join('subject_course', function ($join) {
            $join->on('course.serial_crs', '=', 'subject_course.serial_crs');
        });
        $query->join('subject', function ($join) {
            $join->on('subject_course.serial_sbj', '=', 'subject.serial_sbj');
        });
        $query->join('cat_subject', function ($join) {
            $join->on('subject.serial_csb', '=', 'cat_subject.serial_csb');
        });
        $query->join('academic_area', function ($join) {
            $join->on('cat_subject.serial_aca', '=', 'academic_area.serial_aca');
        });
        $query->where('course.serial_scy','=',$serial_scy);
        $query->where('course.serial_set','=',$serial_set);
        $query->where('course.serial_lvl','=',$serial_lvl);
        $query->where('course.status_crs','=','ACTIVE');
        $query->selectRaw('distinct(academic_area.serial_aca),academic_area.name_aca');

        $optionalSubject = new OptionalSubject();
        $optional = $optionalSubject->newQuery();
        $optional->join('optional_subject_level', function ($join) {
            $join->on('optional_subject.serial_osb', '=', 'optional_subject_level.serial_osb');
        });
        $optional->join('cat_subject', function ($join) {
            $join->on('optional_subject.serial_csb', '=', 'cat_subject.serial_csb');
        });
        $optional->join('academic_area', function ($join) {
            $join->on('cat_subject.serial_aca', '=', 'academic_area.serial_aca');
        });
        $optional->where('optional_subject_level.serial_scy','=',$serial_scy);
        $optional->where('optional_subject_level.serial_set','=',$serial_set);
        $optional->where('optional_subject_level.serial_lvl','=',$serial_lvl);
        $optional->where('optional_subject_level.status_osl','=','ACTIVE');
        $optional->selectRaw('distinct(academic_area.serial_aca),academic_area.name_aca');
        $subjects = $query->union($optional);
        $subjects->orderBy('name_aca');
        return $this->doQueryGet($subjects);
    }

    public function getCourseSubject($serial_scy,$serial_set,$serial_lvl,$serial_aca,$hasNumberGroup){
        $query = $this->newQuery();
        $query->join('class_group', function ($join) {
            $join->on('course.serial_clg', '=', 'class_group.serial_clg');
        });
        $query->join('subject_course', function ($join) {
            $join->on('course.serial_crs', '=', 'subject_course.serial_crs');
        });
        $query->join('subject', function ($join) {
            $join->on('subject_course.serial_sbj', '=', 'subject.serial_sbj');
        });
        $query->join('cat_subject', function ($join) {
            $join->on('subject.serial_csb', '=', 'cat_subject.serial_csb');
        });
        $query->join('academic_area', function ($join) {
            $join->on('cat_subject.serial_aca', '=', 'academic_area.serial_aca');
        });
        $query->leftJoin('subject_group_planning as sgp', function ($join) {
            $join->on('subject_course.serial_sbc', '=', 'sgp.serial_sbc');
            $join->where('sgp.status_sgp', '=', 'ACTIVE');
        });
        $query->where('course.serial_scy','=',$serial_scy);
        $query->where('course.serial_set','=',$serial_set);
        $query->where('course.serial_lvl','=',$serial_lvl);
        if($serial_aca != 'null'){
            $query->where('subject_course.serial_aca','=',$serial_aca);
        }
        if($hasNumberGroup != 'true'){
            $query->where('sgp.serial_sgp','=',null);
        } else {
            $query->where('sgp.serial_sgp','!=',null);
        }
        $query->where('course.status_crs','=','ACTIVE');
        $query->selectRaw("subject_course.serial_sbc,concat(cat_subject.name_csb,' - ',class_group.name_clg) as subject,
        null as serial_osb,sgp.serial_sgp,academic_area.serial_aca,academic_area.name_aca,sgp.number_group_sgp");
        $optionalSubject = new OptionalSubject();
        $optional = $optionalSubject->newQuery();
        $optional->join('optional_subject_level', function ($join) {
            $join->on('optional_subject.serial_osb', '=', 'optional_subject_level.serial_osb');
        });
        $optional->join('cat_subject', function ($join) {
            $join->on('optional_subject.serial_csb', '=', 'cat_subject.serial_csb');
        });
        $optional->join('academic_area', function ($join) {
            $join->on('cat_subject.serial_aca', '=', 'academic_area.serial_aca');
        });
        $optional->leftJoin('subject_group_planning', function ($join) {
            $join->on('optional_subject.serial_osb', '=', 'subject_group_planning.serial_osb');
            $join->where('subject_group_planning.status_sgp', '=', 'ACTIVE');
        });
        $optional->where('optional_subject_level.serial_scy','=',$serial_scy);
        $optional->where('optional_subject_level.serial_set','=',$serial_set);
        $optional->where('optional_subject_level.serial_lvl','=',$serial_lvl);
        if($serial_aca != 'null'){
            $optional->where('optional_subject.serial_aca','=',$serial_aca);
        }
        if($hasNumberGroup != 'true'){
            $optional->where('subject_group_planning.serial_sgp','=',null);
        } else {
            $optional->where('subject_group_planning.serial_sgp','!=',null);
        }
        $optional->where('optional_subject_level.status_osl','=','ACTIVE');
        $optional->selectRaw("null as serial_sbc,concat(cat_subject.name_csb,' - ',cat_subject.code_csb) as subject,
        optional_subject.serial_osb,subject_group_planning.serial_sgp,academic_area.serial_aca,academic_area.name_aca,subject_group_planning.number_group_sgp");
        $subjects = $query->union($optional);
        $subjects->orderBy('subject');
        return $this->doQueryGet($subjects);
    }

    public function getCourseSubjectInfo($serial_scy,$serial_set,$serial_lvl,$serial_aca){
        $query = $this->newQuery();
        $query->join('class_group', function ($join) {
            $join->on('course.serial_clg', '=', 'class_group.serial_clg');
        });
        $query->join('subject_course', function ($join) {
            $join->on('course.serial_crs', '=', 'subject_course.serial_crs');
        });
        $query->join('subject', function ($join) {
            $join->on('subject_course.serial_sbj', '=', 'subject.serial_sbj');
        });
        $query->join('cat_subject', function ($join) {
            $join->on('subject.serial_csb', '=', 'cat_subject.serial_csb');
        });
        $query->join('academic_area', function ($join) {
            $join->on('cat_subject.serial_aca', '=', 'academic_area.serial_aca');
        });
        $query->leftJoin('subject_group_planning as sgp', function ($join) {
            $join->on('subject_course.serial_sbc', '=', 'sgp.serial_sbc');
            $join->where('sgp.status_sgp', '=', 'ACTIVE');
        });
        $query->where('course.serial_scy','=',$serial_scy);
        $query->where('course.serial_set','=',$serial_set);
        $query->where('course.serial_lvl','=',$serial_lvl);
        $query->where('subject_course.serial_aca','=',$serial_aca);
        $query->where('course.status_crs','=','ACTIVE');
        $query->selectRaw("subject_course.serial_sbc,concat(cat_subject.name_csb,' - ',class_group.name_clg) as subject,
        null as serial_osb,sgp.serial_sgp,academic_area.serial_aca,academic_area.name_aca,sgp.number_group_sgp");
        $optionalSubject = new OptionalSubject();
        $optional = $optionalSubject->newQuery();
        $optional->join('optional_subject_level', function ($join) {
            $join->on('optional_subject.serial_osb', '=', 'optional_subject_level.serial_osb');
        });
        $optional->join('cat_subject', function ($join) {
            $join->on('optional_subject.serial_csb', '=', 'cat_subject.serial_csb');
        });
        $optional->join('academic_area', function ($join) {
            $join->on('cat_subject.serial_aca', '=', 'academic_area.serial_aca');
        });
        $optional->leftJoin('subject_group_planning', function ($join) {
            $join->on('optional_subject.serial_osb', '=', 'subject_group_planning.serial_osb');
            $join->where('subject_group_planning.status_sgp', '=', 'ACTIVE');
        });
        $optional->where('optional_subject_level.serial_scy','=',$serial_scy);
        $optional->where('optional_subject_level.serial_set','=',$serial_set);
        $optional->where('optional_subject_level.serial_lvl','=',$serial_lvl);
        $optional->where('optional_subject.serial_aca','=',$serial_aca);
        $optional->where('optional_subject_level.status_osl','=','ACTIVE');
        $optional->selectRaw("null as serial_sbc,concat(cat_subject.name_csb,' - ',cat_subject.code_csb) as subject,
        optional_subject.serial_osb,subject_group_planning.serial_sgp,academic_area.serial_aca,academic_area.name_aca,subject_group_planning.number_group_sgp");
        $subjects = $query->union($optional);
        $subjects->orderBy('subject');
        return $this->doQueryGet($subjects);
    }

    public function getLevelsAcademicArea($serial_scy,$serial_aca){
        $query = $this->newQuery();
        $query->join('subject_course', function ($join) {
            $join->on('course.serial_crs', '=', 'subject_course.serial_crs');
        });
        $query->join('level', function ($join) {
            $join->on('course.serial_lvl', '=', 'level.serial_lvl');
        });
        $query->join('academic_area', function ($join) {
            $join->on('subject_course.serial_aca', '=', 'academic_area.serial_aca');
        });
        $query->where('course.serial_scy','=',$serial_scy);
        $query->where('academic_area.serial_aca','=',$serial_aca);
        $query->where('course.status_crs','=','ACTIVE');
        $query->selectRaw('distinct(level.name_lvl),level.serial_lvl');
        $query->orderBy('name_lvl');
        return $this->doQueryGet($query);
    }

    public function getUnitsPlanning($data){
        $query = $this->newQuery();
        $query->join('subject_course', function ($join) {
            $join->on('course.serial_crs', '=', 'subject_course.serial_crs');
        });
        $query->join('subject_group_planning AS sgp', function ($join) {
            $join->on('subject_course.serial_sbc', '=', 'sgp.serial_sbc');
        });
        $query->leftJoin('subject_year_planning AS syp', function ($join) {
            $join->on('syp.serial_syp', '=', 'sgp.serial_syp');
        });
        $query->with(array('levelPlanningUnit'=>function($query){
            $query->where('status_spu','=','ACTIVE');
        }));
        $query->join('level', function ($join) {
            $join->on('course.serial_lvl', '=', 'level.serial_lvl');
        });
        $query->join('academic_area AS aca', function ($join) {
            $join->on('subject_course.serial_aca', '=', 'aca.serial_aca');
        });
        $query->join('cat_subject AS csb', function ($join) {
            $join->on('aca.serial_aca', '=', 'csb.serial_aca');
        });
        $query->join('subject AS sbj', function ($join) {
            $join->on('csb.serial_csb', '=', 'sbj.serial_csb');
        });
        $query->where('course.serial_scy','=',$data['serial_scy']);
        $query->where('aca.serial_aca','=',$data['serial_aca']);
        if($data['serial_lvl'] != 'all') {
            $query->where('course.serial_lvl','=',$data['serial_lvl']);
        }
        $query->where('course.status_crs','=','ACTIVE');
        $query->selectRaw('distinct(csb.name_csb),level.serial_lvl,level.name_lvl,syp.name_syp');
        $optionalSubject = new OptionalSubject();
        $optional = $optionalSubject->newQuery();
        $optional->join('optional_subject_level', function ($join) {
            $join->on('optional_subject.serial_osb', '=', 'optional_subject_level.serial_osb');
        });
        $optional->join('level AS lvl', function ($join) {
            $join->on('optional_subject_level.serial_lvl', '=', 'lvl.serial_lvl');
        });
        $optional->join('cat_subject', function ($join) {
            $join->on('optional_subject.serial_csb', '=', 'cat_subject.serial_csb');
        });
        $optional->join('academic_area', function ($join) {
            $join->on('cat_subject.serial_aca', '=', 'academic_area.serial_aca');
        });
        $optional->join('subject_group_planning', function ($join) {
            $join->on('subject_group_planning.serial_osb', '=', 'optional_subject.serial_osb');
        });
        $optional->leftJoin('subject_year_planning', function ($join) {
            $join->on('subject_year_planning.serial_syp', '=', 'subject_group_planning.serial_syp');
        });
        $optional->where('optional_subject_level.serial_scy','=',$data['serial_scy']);
        if($data['serial_lvl'] != 'all') {
            $optional->where('optional_subject_level.serial_lvl','=',$data['serial_lvl']);
        }
        $optional->where('academic_area.serial_aca','=',$data['serial_aca']);
        $optional->where('optional_subject_level.status_osl','=','ACTIVE');
        $optional->selectRaw('distinct(cat_subject.name_csb),lvl.serial_lvl,lvl.name_lvl,subject_year_planning.name_syp');
        $subjects = $query->union($optional);
        return $this->doQueryGet($subjects);
    }

}
