<?php

namespace App\Repositories;

use App\Models\IncidenceSetup;
use App\Repositories\BaseRepository;

/**
 * Repositorio para manejo de Incidencia Setup
 * @since   2018-03-17
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-17<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del archivo<br/>
 *          #edit1
 */
class IncidenceSetupRepository extends BaseRepository {

    protected $modelClass = IncidenceSetup::class;

    //Declaración del constructor
    public function __construct(IncidenceSetup $incidenceSetup) {
        $this->incidenceSetup = $incidenceSetup;
    }
  


    /**
     * Método que guarda configuracion
     * @since   2018-03-17
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $data
     * @return  
     * 
     */
    public function saveIncidenceSetup($data) {

        //se realiza la búsqueda para determinar si existe ya configuracion
        $setup = IncidenceSetup::where('serial_ise', '=', $data['serial_ise'])
                ->where('status_ise', '=', 'ACTIVE')
                ->first();

         //se actualiza si se encontró ya una configuracion activa registrada
        if ($setup != null) {
            if (isset($data['_destroy'])) {
                $setup->status_ise = 'INACTIVE';
                $setup->save();
            } else { //UPDATE                   
                $setup = $this->populateForUpdate($setup, $data);
                $setup->save();                
            }
        }else{
            //Validar que no exista ya una incidencia con esa frecuencia (por si estuviera inactiva)
            $setup = IncidenceSetup::where('serial_ica', '=', $data['serial_ica'])
                ->where('frequency_ise', '=', $data['frequency_ise'])
                ->first();

            if($setup != null) {
                $setup = $this->populateForUpdate($setup, $data);
                $setup->status_ise = 'ACTIVE';
                $setup->save();
            }else{
                //Crea nuevo registro
                $setup = new IncidenceSetup;

                $setup->serial_ica = $data['serial_ica'];        
                $setup->frequency_ise = $data['frequency_ise'];                   
                $setup->value_ise = $data['value_ise'];
                $setup->lower_discipline_level_ise = $data['lower_discipline_level_ise'];
                $setup->by_subject_ise = $data['by_subject_ise'];
                $setup->status_ise = 'ACTIVE';
                $setup->save();
            }           

        }             
    }


    /**
     * Método para actualizar objeto para actualizacion
     * @since   2018-03-17
     * @author  Erick Vaca <epvaca@rutatec.com>
     * @return  json resultado
     *
     */
    private function populateForUpdate($setup, $data){
        $setup->serial_ica = $data['serial_ica'];        
        $setup->frequency_ise = $data['frequency_ise'];                   
        $setup->value_ise = $data['value_ise'];
        $setup->lower_discipline_level_ise = $data['lower_discipline_level_ise'];
        $setup->by_subject_ise = $data['by_subject_ise'];          
        return $setup; 
    }


    /**
     * Método para obtener configuracion por anio
     * @since   2018-03-17
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @return  Setup array
     * 
     */
    public function incidenceSetupByYear($serial_scy) {

        $query = $this->newQuery();        
       
        $query->join('incidence_category AS ica','incidence_setup.serial_ica', '=', 'ica.serial_ica');
        $query->where('ica.serial_scy', $serial_scy);
        $query->where('ica.status_ica', 'ACTIVE');    
        $query->where('incidence_setup.status_ise', 'ACTIVE');

        $query->orderBy('incidence_setup.serial_ica');
        $query->orderBy('incidence_setup.frequency_ise','ASC');

         return $this->doQueryGet($query);
    }



    /**
     * Método para obtener configuracion general por serial_ica (Categoria)
     * @since   2018-04-11
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_ica serial de incidence_category
     * @return  Setup array
     * 
     */
    public function incidenceGeneralSetupBySerialIca($serial_ica) {

        $query = $this->newQuery();      
       
        $query->where('serial_ica', $serial_ica);
        $query->where('status_ise', 'ACTIVE');                    
        $query->where('by_subject_ise', '0');
        $query->orderBy('frequency_ise','ASC');

         return $this->doQueryGet($query);
    }


    /**
     * Método para obtener configuracion especifica de materia por serial_ica (Categoria)
     * @since   2018-04-11
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_ica serial de incidence_category
     * @return  Setup array
     * 
     */
    public function incidenceSpecificSubjectSetupBySerialIca($serial_ica) {

        $query = $this->newQuery();      
       
        $query->where('serial_ica', $serial_ica);
        $query->where('status_ise', 'ACTIVE');
        $query->where('by_subject_ise', '1');                    
        $query->orderBy('frequency_ise','ASC');

         return $this->doQueryGet($query);
    }


    /**
     * Método para obtener la maxima configuracion general por serial_ica (Categoria)
     * Maxima configuracion se refiere la mayor frecuencia configurada general
     * @since   2018-04-11
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_ica serial de incidence_category
     * @return  Setup array
     * 
     */
    public function maxIncidenceGeneralSetupBySerialIca($serial_ica) {

        $query = $this->newQuery();      
       
        $query->where('serial_ica', $serial_ica);
        $query->where('status_ise', 'ACTIVE');
        $query->where('by_subject_ise', '0');                    
        $query->orderBy('frequency_ise','DESC');

         return $this->doQueryFirst($query);
    }


    /**
     * Método para obtener la maxima configuracion especifica de materia por serial_ica (Categoria)
     * Maxima configuracion se refiere la mayor frecuencia configurada especifica
     * @since   2018-04-11
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_ica serial de incidence_category
     * @return  Setup array
     * 
     */
    public function maxIncidenceSpecificSubjectSetupBySerialIca($serial_ica) {

        $query = $this->newQuery();      
       
        $query->where('serial_ica', $serial_ica);
        $query->where('status_ise', 'ACTIVE');
        $query->where('by_subject_ise', '1');
        $query->orderBy('frequency_ise','DESC');

         return $this->doQueryFirst($query);
    }

}
